# Generated by Django 4.1 on 2022-12-06 20:56

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cities", "0011_auto_20180108_0706"),
    ]

    operations = [
        migrations.AlterField(
            model_name="country",
            name="neighbours",
            field=models.ManyToManyField(
                related_name="_cities_country_neighbours_+",
                to=settings.CITIES_COUNTRY_MODEL,
            ),
        ),
    ]
