# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-26 03:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cities", "0007_add_currency_and_postal_code_fields_to_country_model"),
    ]

    operations = [
        migrations.AddField(
            model_name="district",
            name="code",
            field=models.CharField(
                blank=True, db_index=True, max_length=200, null=True
            ),
        ),
    ]
